#!/usr/bin/env python
# -*- coding: utf-8 -*-
#############################################################################
# Eole NG - 2007  
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr 
#  
# prelude_server.py
#  
# Serveur XML-RPC pour permettre l'utilisation à distance de prelude-adduser
# Nécessite que la machine soit enregistrée sur zephir.
#       
#############################################################################

import os, subprocess, signal
import base64
from twisted.web import xmlrpc, server
from twisted.application import internet
from twisted.internet import reactor
# options ligne de commande
from twisted.python import usage, log 
# imports eole
import config
from creole.cert import ServerContextFactory
try:
    from zephir.zephir_conf.zephir_conf import id_serveur, adresse_zephir
    from zephir.lib_zephir import xmlrpclib, TransportEole
    registered = True
except:
    registered = False

# initialisations pour la localisation
APP = 'eole-prelude-server'
import locale, gettext
locale.setlocale(locale.LC_ALL, config.LC_ALL)
gettext.bindtextdomain (APP)
gettext.textdomain (APP)
# installons la fonction _() dans les builtins
gettext.install (APP, unicode=0)

try:
    _ = _
except NameError:
    _ = unicode

#registerer = None

NOBODY_GID = NOBODY_UID = 65534

class Options(usage.Options):
    optParameters = [["port", "p", config.PORT_WEB,
                      "Numero de port pour le serveur web."]]

class PreludeRpc(xmlrpc.XMLRPC):
    """Serveur rpc pour lancement à distance de prelude-manager"""

    def _drop_privileges(self):
        #drop de privileges vers nobody/nobody
        os.setegid(NOBODY_GID)
        os.seteuid(NOBODY_UID)

    def _gain_privileges(self):
        #on reprend nos privileges initiaux
        os.seteuid(self.UID)
        os.setegid(self.GID)

    def __init__(self):
        #sauvegarde de notre UIG/GID courant
        self.registerer = None
        self.UID = os.getuid()
        self.GID = os.getgid()
        xmlrpc.XMLRPC.__init__(self)
        self.key = None
        self.key_file = '/var/spool/uucp/.ssh/id_rsa.pub'

    def startFactory(self):
        #sauvegarde de notre UIG/GID courant
        self._drop_privileges()
        server.Site.startFactory(self)

    def stopFactory(self):
        # reprise des droits pour supprimer le fichier de PID
        self._gain_privileges()
        server.Site.stopFactory(self)

    def check_key(self, key):
        key = key.split('ssh-rsa')[1].strip()
        # lecture de la clé rsa zephir
        if self.key == None:
            self._gain_privileges()
            if os.path.isfile(self.key_file):
                try:
                    self.key = file(self.key_file).read().split('ssh-rsa')[1].strip()
                except:
                    pass
            self._drop_privileges()
        if self.key != None and key == self.key:
            return True
        return False

    def xmlrpc_check_perms(self, user, passw):
        return self.check_auth(user, passw)

    def check_auth(self, login, passwd):
        if registered == True:
            zephir = xmlrpclib.Server('https://%s:%s@%s:7080' % (login, passwd, adresse_zephir))
            try:
                code, res = zephir.serveurs.global_status(id_serveur)
                code, perms = zephir.get_permissions(login)
                if code == 1 and res != []:
                    # droits sur les fonctions prelude nécessaires
                    if 10 in perms:
                        return True
            except Exception, e:
                # pb d'authentification zephir ou pas accès à ce serveur
                pass
        # pas authentifié ou pas enregistré
        return False

    def _kill_adduser(self):
        """supprime les processus prelude-adduser existants"""
        log.msg(_("checking remaining adduser instances"))
        self._gain_privileges()
        if self.registerer and self.registerer.poll() == None:
            # instance non terminée, on arrete le serveur
            reg_pid = self.registerer.pid
            os.kill(reg_pid, signal.SIGTERM)
            del (self.registerer)
            self.registerer = None
            log.msg(_("registration-server instance without response (ending process %s)") % str(reg_pid))
        self.kill_callb = None
        self._drop_privileges()

    def _close_adduser(self):
        """supprime les objets liés au précédent lancement de registration-server
        (évite les processus zombie) et annule le callback de timeout
        """
        if self.kill_callb:
            log.msg(_("registration successful (cleaning up process %s)") % str(self.registerer.pid))
            self.kill_callb.cancel()
            del(self.kill_callb)
        del(self.registerer)

    def _get_min_env(self):
        """prépare un environnement minimal à passer à un processus fils
        """
        variables  = ["LANGUAGE", "LANG", "PATH", "HOME", "DISPLAY"]
        env = {}
        for name in variables:
           value = os.getenv(name)
           if value is not None:
              env[name] = value
        return env


    def xmlrpc_register_server(self, key, register_pass):
        """fonction pour l'enregistrement d'une sonde
        @param key: cle publique de serveur envoyée par zephir
        @param register_pass: mot de passe utilisé au lancement de registration-server
        renvoie : 1 si lancement ok
                  0 si lancement impossible
                 -1 si le serveur est déjà en attente d'un enregistrement
        """
        if self.registerer:
            if self.registerer.poll() == None:
                # un serveur d'enregistrement est encore en attente
                return -1
            else:
                self._close_adduser()
                # annulation
        # vérification de l'authentification sur zephir
        log.msg(_("launching prelude registerer server with password : "), register_pass)
        key = base64.decodestring(key)
        if self.check_key(key):
            # authentification ok : lancement de adduser
            #cmd_adduser = """/usr/bin/prelude-admin registration-server prelude-manager --passwd "%s" --no-confirm &""" % register_pass
            cmd_adduser = ["/usr/bin/prelude-admin", "registration-server", "prelude-manager", "--passwd", register_pass, "--no-confirm"]
            self._gain_privileges()
            self.registerer = subprocess.Popen(cmd_adduser, stdin=subprocess.PIPE, stdout=subprocess.PIPE, env=self._get_min_env())
            self._drop_privileges()
            # on tue le processus dans 20 secondes si aucun enregistrement n'a eu lieu
            self.kill_callb = reactor.callLater(20, self._kill_adduser)
            return 1
        else:
            log.msg(_("bad key sent for server registration"))
        return 0

def makeService(config):
    # port TCP à écouter
    port = int(config["port"])
    serv = server.Site(PreludeRpc())
    return internet.SSLServer(port, serv, ServerContextFactory())

